--[[
This class controls the arrangement of the ActionButtonNone and ActionButton
instances for a UnitGroupHeader.

There are one or more IActionSetLayout instances allocated for each
UnitGroupHeader.  How many depends on whether the layout is changed while
in-game.  The only layouts allocated are ones that were selected in-game.  Each
UnitGroupHeader keeps track of the active ActionSetLayout to use.
--]]
uabPyramidLayoutManager.Layout = uab.mixins.aceOO.Class(IActionSetLayout)

function uabPyramidLayoutManager.Layout.prototype:init(context, settings)
	uabPyramidLayoutManager.Layout.super.prototype.init(self)
	
    self.config = uabPyramidLayoutManager.LayoutConfig:new(self, settings)
    self.context = context
    self.name = "uab.Pyramid_"..context
end

function uabPyramidLayoutManager.Layout.prototype:LayoutSettingsChanged()
    uabLayoutManager:LayoutSettingsChanged()
end

function uabPyramidLayoutManager.Layout.prototype:ArrangeButtons(cancelbtn, actionbtns, states)
    uab:Debug(self.name .. " : ArrangeButtons")
    local found

    --[[
    First, reset all positioning information
    --]]
    self:FormatCancelBtn(cancelbtn)

    for idx = 1, 24, 1 do
        if actionbtns[idx] then
            self:FormatActionBtn(actionbtns[idx])
            if not found then
                found = actionbtns[idx].isenabled
            end
        end
    end

    --[[
    Now, position the cancel button centered relative to the header (the
    parent of the cancel button)
    --]]
    if not self.config.settings.hidecancel and found then
        cancelbtn:AddShowStates(states)
        cancelbtn:SetPointForStates(states, "CENTER", 0, 0)
    end

    self:SetupRowInfo()

    for idx = 1, self.rowinfo.numrows, 1 do
        self:PositionRow(states, idx, cancelbtn:Frame(), actionbtns)
    end
end

function uabPyramidLayoutManager.Layout.prototype:SetupRowInfo()
    if not self.rowinfo then
        self.rowinfo = {}
    end

    self.rowinfo.numrows = 1
    if not self.rowinfo.rowcount then
        self.rowinfo.rowcount = {}
    else
        for idx, val in pairs(self.rowinfo.rowcount) do
            self.rowinfo.rowcount[idx] = nil
        end
    end

    self.rowinfo.rowcount[1] = self.config.settings.minRowCount

    local remaining = 24 - self.config.settings.minRowCount
    local idx = 2
    local rowcount = self.config.settings.minRowCount

    while remaining > 0 do
        rowcount = rowcount + self.config.settings.rowChangeAmount
        if rowcount > remaining then
            rowcount = remaining
        end
        self.rowinfo.rowcount[idx] = rowcount
        idx = idx + 1
        remaining = remaining - rowcount
        self.rowinfo.numrows = self.rowinfo.numrows + 1
    end

    if self.config.settings.expand ~= "true" then
        local reverse = {}

        for revidx = 1, self.rowinfo.numrows, 1 do
            reverse[revidx] = self.rowinfo.rowcount[(self.rowinfo.numrows-revidx)+1]
        end

        for index, val in pairs(reverse) do
            self.rowinfo.rowcount[index] = val
        end
    end
end

function uabPyramidLayoutManager.Layout.prototype:PositionRow(states, row, cancel, actionbtns)
    local startbutton = self:FirstButtonForRow(row)
    local numbuttons = self.rowinfo.rowcount[row]
    local endbutton = startbutton + numbuttons - 1

    local hchange
    local vchange
    local hstart
    local vstart
    local hspacing
    local vspacing

    if self.config.settings.extendDirection == uab.constants.grow.down or
        self.config.settings.extendDirection == uab.constants.grow.up then
        hspacing = self.config.settings.rowSpacing
        vspacing = self.config.settings.extendSpacing
    else
        hspacing = self.config.settings.extendSpacing
        vspacing = self.config.settings.rowSpacing
    end
    
    if self.config.settings.extendDirection == uab.constants.grow.down then
        local buttonspace = (uab.constants.buttonWidth + hspacing) * numbuttons

        hchange = (uab.constants.buttonWidth + hspacing)
        hstart = -((buttonspace - (uab.constants.buttonWidth + hspacing)) / 2)
        if self.config.settings.fillDirection == uab.constants.grow.left then
            hchange = hchange * -1
            hstart = hstart * -1
        end

        vchange = 0
        vstart = -((uab.constants.buttonHeight + vspacing) * row)
    elseif self.config.settings.extendDirection == uab.constants.grow.up then
        local buttonspace = (uab.constants.buttonWidth + hspacing) * numbuttons

        hchange = (uab.constants.buttonWidth + hspacing)
        hstart = -((buttonspace - (uab.constants.buttonWidth + hspacing)) / 2)
        if self.config.settings.fillDirection == uab.constants.grow.left then
            hchange = hchange * -1
            hstart = hstart * -1
        end

        vchange = 0
        vstart = (uab.constants.buttonHeight + vspacing) * row
    elseif self.config.settings.extendDirection == uab.constants.grow.left then
        local buttonspace = (uab.constants.buttonHeight + vspacing) * numbuttons

        hchange = 0
        hstart = -((uab.constants.buttonWidth + hspacing) * row)

        vchange = (uab.constants.buttonHeight + vspacing)
        vstart = -((buttonspace - (uab.constants.buttonHeight + 1)) / 2)
        if self.config.settings.fillDirection == uab.constants.grow.down then
            vchange = vchange * -1
            vstart = vstart * -1
        end
    elseif self.config.settings.extendDirection == uab.constants.grow.right then
        local buttonspace = (uab.constants.buttonHeight + vspacing) * numbuttons

        hchange = 0
        hstart = (uab.constants.buttonWidth + hspacing) * row

        vchange = (uab.constants.buttonHeight + vspacing)
        vstart = -((buttonspace - (uab.constants.buttonHeight + vspacing)) / 2)
        if self.config.settings.fillDirection == uab.constants.grow.down then
            vchange = vchange * -1
            vstart = vstart * -1
        end
    end

    local hpos = hstart
    local vpos = vstart

    for idx = startbutton, endbutton, 1 do
        if actionbtns[idx] and actionbtns[idx].isenabled then
            actionbtns[idx]:AddShowStates(states)
            actionbtns[idx]:SetPointForStates(states, "CENTER", hpos, vpos)
        end
        hpos = hpos + hchange
        vpos = vpos + vchange
    end
end

function uabPyramidLayoutManager.Layout.prototype:FirstButtonForRow(row)
    local btnpos = 1
    for idx = 1, row-1, 1 do
        btnpos = btnpos + self.rowinfo.rowcount[idx]
    end
    
    return btnpos
end

function uabPyramidLayoutManager.Layout.prototype:GetConfig()
	return self.config
end

function uabPyramidLayoutManager.Layout.prototype:SetSettings(settings)
    self.config:SettingsChanged(settings)
end

function uabPyramidLayoutManager.Layout.prototype:FormatCancelBtn(cancelbtn)
    -- Nothing to do here at this time.  Would be a good idea to resize
    -- the cancel button frame, hide the text frame for the button frame
end

function uabPyramidLayoutManager.Layout.prototype:FormatActionBtn(btn)
    -- Nothing to do here at this time.  Would be a good idea to resize
    -- the cancel button frame, hide the text frame for the button frame
end
